(*

  application: image filtering
  state: experimental
  revision: $Id: defs.mli,v 1.3 2005/06/28 07:33:08 tobias Exp $
 
  Metaprogramming Project
  Authors: Christoph A. Herrmann and Tobias Langhammer
  University of Passau, Department of Informatics, Chair of Programming

 Copyright (C) 2004    University of Passau, Department of Informatics
 
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
 
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*)

exception Lookup_error of string
exception Type_error of string


(******* datatypes and functions for AST **************)
type value  = 
    VInt of int
  | VFloat of float
  | VBool of bool
  | VMatrix of float array array


type dtype = Bool | Int | Float | Matrix


type unOp 
 = NegI | NegF
 | Not
 | AbsI | AbsF
 | Exp | Log | Sqrt | Sin | Cos | Tan | ASin | ACos | ATan

type binOp 
 = AddI | SubI | MulI | DivI | PowI | MinI | MaxI | ModI  
 | AddF | SubF | MulF | DivF | PowF | MinF | MaxF
 | EqB  | UEqB | AndB | OrB
 | EqI  | UEqI | LtI  | LeI  | GtI  | GeI
 | EqF  | UEqF | LtF  | LeF  | GtF  | GeF


type op = 
    C of value 
  | V of string
  | Image
  | Int2Float 
  | Floor
  | UnOp of unOp
  | BinOp of binOp        
  | If                    (* x:bool, y:'a, z:'a *)
  | Let of string         (* x:'a,   y:'b       *)
  | IndexMatrix of string (* x:int,  y: int     *)
  | Sum of string        (* low:int, high:int, exp:int *)

type exp = { 
  dtype:dtype;
  op:op;
  args:exp list;
  expand:int 
}

type top_exp = { common_bind:(string * exp) list; channels: exp list }
val create_exp: dtype -> op -> exp list -> exp
val exp_of_value: value -> exp


val unC : exp -> value option

val head : ?info:string ->  'a list -> 'a
val head2 : 'a list -> 'a * 'a
val head3 : 'a list -> 'a * 'a * 'a

val type_of_binop : binOp -> dtype

val string_of_value : value -> string
val string_of_dtype : dtype -> string
val string_of_binOp : binOp -> string
val string_of_unOp  : unOp -> string
val string_of_op : op -> string
val string_of_exp : exp -> string
val print_exp : exp -> unit

(******* datatype an functions for environment ********)
type 'a environment = string -> 'a option
val extEnv : string * 'a -> 'a environment -> 'a environment
val newEnv : ('a * 'b) list -> 'a -> 'b option
val extEnvPairs : ('a * 'b) list -> ('a -> 'b option) -> 'a -> 'b option
val emptyEnv : 'a environment 
val lookup : string -> 'a environment -> 'a option
val lookup_excp : string -> 'a environment -> 'a

val static_env : (string * dtype * value option) list
val dynamic_env : (string * dtype * value option) list

val size_top_exp : top_exp -> int
val size_exp : exp -> int
