(* vim: ft=ocaml 

  NOTE: this file was generated from a .m4 template

  application: image filtering
  state: experimental
  revision: $Id: filt_config.ml,v 1.3 2005/06/29 15:52:25 tobias Exp $
 
  Metaprogramming Project
  Authors: Christoph A. Herrmann and Tobias Langhammer
  University of Passau, Department of Informatics, Chair of Programming

 Copyright (C) 2004    University of Passau, Department of Informatics
 
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
 
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*)

ifdef(`__UNSTAGED',` include(`config_unstaged.m4')',` include(`config_staged.m4')')

open Printf
open Ppm
open Arg

type mode = Normal | Benchmark of (int * (int -> int))

let set_string r = String (fun s -> r := Some s)

(* variables for keeping command line flags *)
let e = ref None
let f = ref None
let i = ref None
let o = ref None
let m = ref 0
let mt = ref 0
let mb = ref 0
let ml = ref 0
let mr = ref 0
let v = ref 1
let n = ref 1
let ns = ref None
let indep = ref true
let tn = ref 1
let ts = ref false
let tc = ref false
let tb = ref false
let te = ref false

ifdef(`__UNSTAGED',`
let staged = ref false
',`
let staged = ref true
')
let simplify = ref true
let max_unrolling = ref 2000
let bench = ref false
let bi = ref 1
let bw = ref 50.0
let bs = ref 1.0
let bb = ref 10.0
let bf = ref 0.0

let arg_list = [
  "-e", set_string e, "<string>    set filter expression at command line";
  "-f", set_string f, "<file>      file with filter expression";
  "-i", set_string i, "<img>       input ppm image";
  "-o", set_string o, "<img>       output ppm image";
  "-m", Int (fun m -> List.iter (fun v -> v := m) [mt;mb;ml;mr]),
                      "<size>      set all margins";
  "-mt", Set_int mt,   "<size>     top margin";
  "-mb", Set_int mb,   "<size>     bottom margin";
  "-ml", Set_int ml,   "<size>     left margin";
  "-mr", Set_int mr,   "<size>     right margin";
  "-v" , Set_int v,   "<n>         level of verbosity (default: 1)";
  "-n" , Set_int n,   "<n>         number of iterations (default: 1)";
  "-ns", set_string ns,
                       "<suffix>   write intermediate files of iteration to <n><suffix>";
  "-ni", Set indep,    "           independent filter on original input (default)";
  "-nc", Clear indep,  "           consecutive filter on previous iteration";
  "-tn", Set_int tn,   "<n>        number of timings to be done";
  "-t", Unit (fun () -> ts := true;tc := true;tb := true;te := true),
                      "            show all times for run";
  "-ts", Set ts,       "           show times for simplification";
  "-tc", Set tc,       "           show times for code generation";
  "-tb", Set tb,       "           show times for bang (.!) operator";
  "-te", Set te,       "           show times for filter execution";
  "--bench", Set bench,    "       enter benchmark mode";
  "-bi", Set_int bi,  "<i>             benchmarks for following side lengths:";
  "-bf", Set_float bf, "<f>             for each k in [0, ..., <i>] {";
  "-bb", Set_float bb, "<b>               side length = <f> * <b>^k + <s> * <k> + <w>";
  "-bs", Set_float bs, "<s>             }";
  "-bw", Set_float bw, "<w>           ";
ifdef(`__UNSTAGED',`',`
    "--staged", Set staged,     "      staging optimization (default)";
')
    "--unstaged", Clear staged, "    staging optimization off";
    "--simplify", Set simplify,  "    expression simplification optimization (default)";
    "--nosimplify", Clear simplify,"  expression simplification optimization off";
    "-u", Set_int max_unrolling, "<n>      maximal extent of unrolled code";
]

let anon_fun v = raise (Failure ("unknown argument "^v))
let usage_msg = "image filter usege:" 

let string_of_file f = begin
  let ic = open_in f in
  let buf = Buffer.create 100 in
  begin try while true  do
    Buffer.add_string buf " ";
    Buffer.add_string buf (input_line ic)
  done with End_of_file -> () end;
  close_in ic;
  Buffer.contents buf
end



type compilation = (int * int) -> Ppm.point
type config = { 
  mode:mode;
  ic:in_channel;
  oc:out_channel;
  osuffix:string option;
  iter_indep: bool;
  staged:bool;
  simplify:bool;
  max_unrolling: int;
  simplifyer: Ppm.image -> int -> Defs.top_exp -> Defs.top_exp;
  compiler: Ppm.image -> int -> Defs.top_exp -> compilation;
  time_simplify : bool;
  time_codegen : bool;
  time_bang : bool;
  time_exec : bool;
  iter:int;
  timings_iter:int;
  margintop:int;
  marginbottom:int;
  marginleft:int;
  marginright:int;
  print: int -> string Lazy.t -> unit;
  prerr: int -> string Lazy.t -> unit;
  formula:string;
  label:string;
}



let get () = (* unit -> config *)
  begin
    Arg.parse arg_list anon_fun usage_msg;

    (* build configuration from parsed information *)
    let rec power b = function 
      | 0 -> 1.0
      | e -> b *. power b (e - 1)
    in
    let mode = if !bench then
      Benchmark (!bi, fun (k:int) ->
        int_of_float (!bf *. power !bb k +. !bs *. float_of_int k +. !bw))
    else Normal in
    let formula:string = match !e,!f with
      | Some e,_ -> e
      | _, Some f -> string_of_file f
      | None,None -> "[3 channels: image(row,col,current)]"
    in
    let ic, label = match !bench, !i with
      | true , _ -> stdin, "<stdin>"
      | _, Some "-" -> stdin, "<stdin>"
      | _, Some i -> open_in i, i
      | _, None -> raise (Failure "missing infile")
    in
    let oc = match !bench, !o with
      | true, _ -> open_out "/dev/null"
      | _, Some "-" -> stdout
      | _, Some o -> open_out o
      | _, None -> raise (Failure "missing outfile")
    in
    let print level msg = 
      if level <= !v then (print_string (Lazy.force msg);flush stdout) in
    let prerr level msg = 
      if level <= !v then (prerr_string (Lazy.force msg);flush stdout) in
    
    let simplifyer =
      if !simplify then
        fun img iter ast -> 
          Staticeval.simpl (!max_unrolling) ast img.width img.height iter
      else
        fun _ _ ast -> ast
    in
    let compiler =
      if !staged then begin
ifdef(`__UNSTAGED',`
        raise (Failure "staging not allowed for this build")
',`
        print 2 (lazy "using staging\n");
        fun img iter ast -> Stagedinter.compile !tc !tb iter ast img
')
     end else begin
        print 2 (lazy "using no staging\n");
        fun img iter ast -> Unstagedinter.compile !tc !tb iter ast img
     end
    in

    {
      mode = mode;
      ic = ic;
      oc = oc;
      osuffix = !ns;
      iter = !n;
      timings_iter = !tn;
      iter_indep = !indep;
      staged = !staged;
      simplify = !simplify;
      max_unrolling = !max_unrolling;
      simplifyer = simplifyer;
      compiler = compiler;
      time_simplify = !ts;
      time_codegen = !tc;
      time_bang = !tb;
      time_exec = !te;
      margintop = !mt;
      marginleft = !ml;
      marginright = !mr;
      marginbottom = !mb;
      print = print;
      prerr = prerr;
      formula = formula; 
      label = label;
    } 
  end

let finalize cfg = 
  begin
    if cfg.ic <> stdin then close_in cfg.ic;
    if cfg.oc <> stdout then close_out cfg.oc;
  end   

