(*

  application: image filtering
  state: experimental
  revision: $Id: filt_lexer.mll,v 1.1.1.1 2005/06/17 08:56:18 tobias Exp $
 
  Metaprogramming Project
  Authors: Christoph A. Herrmann and Tobias Langhammer
  University of Passau, Department of Informatics, Chair of Programming

 Copyright (C) 2004    University of Passau, Department of Informatics
 
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
 
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*)

{
  open Filt_parser        (* The type token is defined in Filt_parser.mli *)
  exception Eof
}
let alpha = ['a'-'z' 'A'-'Z']
let alphanum = ['a'-'z' 'A'-'Z' '0'-'9']
let varchar = ['a'-'z' 'A'-'Z' '0'-'9' '_']
let num = ['0'-'9']
rule token = parse
    [' ' '\t']     { token lexbuf }     (* skip blanks *)
  | eof            { EOL }
  | (num+['.']num*) { FLOAT(float_of_string (Lexing.lexeme lexbuf)) }
  | (num+)         { INT(int_of_string (Lexing.lexeme lexbuf)) }
  | '+'            { PLUS }
  | '-'            { MINUS }
  | '*'            { TIMES }
  | "**"           { POWER }
  | '/'            { DIV }
  | '%'            { MOD }
  | '('            { LPAREN }
  | ')'            { RPAREN }
  | '['            { LBRACK }
  | ']'            { RBRACK }
  | ','            { COMMA }
  | ':'            { COLON }
  | ';'            { SEMICOL }
  | '|'            { SEP }
  | "floor"        { FLOOR }
  | "max"          { MAX }
  | "min"          { MIN }
  | "abs"          { ABS }
  | "sin"          { SIN }
  | "cos"          { COS }
  | "tan"          { TAN }
  | "exp"          { EXP }
  | "log"          { LOG }
  | "sqrt"         { SQRT }
  (*
  | "red"          { RED }
  | "green"        { GREEN }
  | "blue"         { BLUE }
  | "gray"         { GRAY }
  | "grey"         { GRAY }
  | "color"        { COLOR }
  | "colour"       { COLOR }
  *)
  | "image"        { IMAGE }
  | "if"           { IF }
  | "then"         { THEN }
  | "else"         { ELSE }
  | "let"          { LET }
  | "in"           { IN }
  | "from"         { FROM }
  | "to"           { TO }
  | "sum"          { SUM }
  | "of"           { OF }
  | '<'            { LESS }
  | "<="           { LESSEQ }
  | '>'            { GREATER }
  | ">="           { GREATEQ }
  | '='            { EQ }
  | "<>"           { UEQ }
  | "not"          { NOT }
  | "&&"           { AND }
  | "||"           { OR }
  | "true"         { TRUE }
  | "false"        { FALSE }
  | "channels"     { CHANNELS }
  | (alpha varchar*) { VAR (Lexing.lexeme lexbuf) }
  | eof            { raise Eof }
