(*

  application: image filtering
  state: experimental
  revision: $Id: ppm.mli,v 1.1.1.1 2005/06/17 08:56:18 tobias Exp $
 
  Metaprogramming Project
  Authors: Christoph A. Herrmann and Tobias Langhammer
  University of Passau, Department of Informatics, Chair of Programming

 Copyright (C) 2004    University of Passau, Department of Informatics
 
  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.
 
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*)
type point = int array
val redChannel : int
val greenChannel : int
val blueChannel : int

val string_of_channel : int -> string

type image = 
  {
    height: int; 
    width: int ; 
    maxval: int; 
    raster: ((int array) array) array }

val black : point

val gray_of : int -> point
val create : int -> int -> int -> point -> image
val init : int -> int -> int -> (int -> int->point) -> image
val string_of_point : point -> string
val string_of_image : image -> string
val get_point : image -> int -> int -> point
val get_point_red : image -> int -> int -> int
val get_point_green : image -> int -> int -> int
val get_point_blue : image -> int -> int -> int
val set_point : image -> int -> int -> point -> unit
val set_rgb : image -> int -> int -> int -> int -> int -> unit

(* returns (height,width,maxval) *)
val get_dims: image -> int * int * int

exception UnexpectedInput of string

val input_ppm : in_channel -> image
val output_ppm : out_channel -> image -> unit

val load_ppm_file : string -> image
val store_ppm_file : string -> image -> unit



